package lars.tools;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.MouseInfo;
import java.awt.PointerInfo;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Observable;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.*;
//import java.util.Timer;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import lars.Globals;
import lars.ProgramStatement;
import lars.loongarch.hardware.AccessNotice;
import lars.loongarch.hardware.AddressErrorException;
import lars.loongarch.hardware.Memory;
import lars.loongarch.hardware.MemoryAccessNotice;
import lars.loongarch.instructions.BasicInstruction;
import lars.loongarch.instructions.BasicInstructionFormat;
import lars.venus.RunAssembleAction;
import lars.venus.RunBackstepAction;
import lars.venus.RunStepAction;
import lars.venus.VenusUI;

public class LoongArchXray extends AbstractLarsToolAndApplication{
	private static final long serialVersionUID = -1L;
    private static String heading =  "LoongArch X-Ray - Animation of LoongArch Datapath";
	private static String version = " Version 2.0";
	
	protected Graphics g;   
	protected int lastAddress = -1; //address of instruction in memory
	protected JLabel label; 
	private Container painel = this.getContentPane();

	private DatapathAnimation datapathAnimation;   //class panel that runs datapath animation.

    private GraphicsConfiguration gc;
    private BufferedImage datapath;
    private String instructionBinary;
    
    //Components to add menu bar in the plugin window.
    private JButton  Assemble,  Step, runBackStep;
    private Action runAssembleAction, runStepAction, runBackstepAction;
    
    private VenusUI mainUI;
    private JToolBar toolbar;
    private Timer time;
    
    public LoongArchXray(String title, String heading) {
        super(title,heading);
     }
   	 
   	 /**
   	  *  Simple constructor, likely used by the LoongArchXray menu mechanism
   	  */
      public LoongArchXray() {
        super (heading+", "+version, heading);
     }
   		 
   		 
      /**
   	  *  Required method to return Tool name.  
   	  *  @return  Tool name.  LARS will display this in menu item.
   	  */
      public String getName() {
        return "LoongArch X-Ray";
     }

      /**
    	  *  Overrides default method, to provide a Help button for this tool/app.
    	  */
        protected JComponent getHelpComponent() {
          final String helpContent = 
                              "This plugin is used to visualizate the behavior of loongarch processor using the default datapath. \n"+
                              "It reads the source code instruction and generates an animation representing the inputs and \n"+
             						"outputs of functional blocks and the interconnection between them.  The basic signals \n"+
             						"represented are, control signals, opcode bits and data of functional blocks.\n"+
             						"\n"+
             						"Besides the datapath representation, information for each instruction is displayed below\n"+
             						"the datapath. That display includes opcode value, with the correspondent colors used to\n"+
             						"represent the signals in datapath, mnemonic of the instruction processed at the moment, registers\n"+
             						"used in the instruction and a label that indicates the color code used to represent control signals\n" +
             						"\n"+
										"To see the datapath of register bank and control units click inside the functional unit.\n\n" +
             						"Version 2.0\n" + 
             						"Developed by Mrcio Roberto, Guilherme Sales, Fabr�cio Vivas, Fl�vio Cardeal and F�bio L�cio\n" +
             						"Contact Marcio Roberto at marcio.rdaraujo@gmail.com with questions or comments.\n"
             						;
          JButton help = new JButton("Help");
          help.addActionListener(
                 new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                      JOptionPane.showMessageDialog(theWindow, helpContent);
                   }
                });		
          return help;  
       }
     /**
   	 *  Implementation of the inherited abstract method to build the main 
   	 *  display area of the GUI.  It will be placed in the CENTER area of a 
   	 *  BorderLayout.  The title is in the NORTH area, and the controls are 
   	 *  in the SOUTH area.
   	 */
      protected JComponent buildAnimationSequence(){
   	   JPanel image =  new JPanel(new GridBagLayout());
   	   return image;
      }

      // Insert image in the panel and configure the parameters to run animation.
      protected JComponent buildMainDisplayArea() {
   	   mainUI = Globals.getGui();
   	   this.createActionObjects();
   	   toolbar= this.setUpToolBar();
   	  
//   	   JPanel jp = new JPanel(new FlowLayout(FlowLayout.LEFT));
   	    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
   	    gc = ge.getDefaultScreenDevice().getDefaultConfiguration();
   	    try {
   			  BufferedImage im =  ImageIO.read(
   					  getClass().getResource(Globals.imagesPath+"DatapathLA.png") );
   	
   		       int transparency = im.getColorModel().getTransparency();
   		       datapath =  gc.createCompatibleImage( im.getWidth(), im.getHeight(),
   				                        transparency );
   		     	 
   		       Graphics2D g2d = datapath.createGraphics();  // graphics context
   		       g2d.drawImage(im,0,0,null);
   		       g2d.dispose();
   		      
   	     } 
   	     catch(IOException e) {
   	       System.out.println("Load Image error for " +
   	    		   getClass().getResource(Globals.imagesPath+"DatapathLA.png") + ":\n" + e);
   	       		   e.printStackTrace();
   	     }
   	     System.setProperty("sun.java2d.translaccel", "true"); 
            ImageIcon icon = new ImageIcon(getClass().getResource(Globals.imagesPath+"DatapathLA.png"));
            Image im = icon.getImage();
            icon = new ImageIcon(im);

            JLabel label = new JLabel(icon);                
            painel.add(label, BorderLayout.WEST);
            painel.add(toolbar, BorderLayout.NORTH);
   	     this.setResizable(false); 
   	     return (JComponent) painel;
    }
      
      protected JComponent buildMainDisplayArea(String figure) {
      	   mainUI = Globals.getGui();
      	   this.createActionObjects();
      	   toolbar= this.setUpToolBar();
      	  
//      	   JPanel jp = new JPanel(new FlowLayout(FlowLayout.LEFT));
      	    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
      	    gc = ge.getDefaultScreenDevice().getDefaultConfiguration();
      	    try {
      			  BufferedImage im =  ImageIO.read( 
      					  getClass().getResource(Globals.imagesPath+figure) );
      	
      		       int transparency = im.getColorModel().getTransparency();
      		       datapath =  gc.createCompatibleImage( im.getWidth(), im.getHeight(),
      				                        transparency );
      		     	 
      		       Graphics2D g2d = datapath.createGraphics();  // graphics context
      		       g2d.drawImage(im,0,0,null);
      		       g2d.dispose();
      		      
      	     } 
      	     catch(IOException e) {
      	       System.out.println("Load Image error for " +
      	    		   getClass().getResource(Globals.imagesPath+figure) + ":\n" + e); 
      	       		   e.printStackTrace();
      	     }
      	     System.setProperty("sun.java2d.translaccel", "true"); 
               ImageIcon icon = new ImageIcon(getClass().getResource(Globals.imagesPath+figure));
               Image im = icon.getImage();
               icon = new ImageIcon(im);
               
               JLabel label = new JLabel(icon);                
               painel.add(label, BorderLayout.WEST);
               painel.add(toolbar, BorderLayout.NORTH);
      	     this.setResizable(false); 
      	     return (JComponent) painel;
       }  
    
   	protected void addAsObserver() {
   		addAsObserver(Memory.textBaseAddress, Memory.textLimitAddress);
   	}
   	
   	//Function that gets the current instruction in memory and start animation with the selected instruction.
   	protected void processLoongArchUpdate(Observable resource, AccessNotice notice) {

   		if (!notice.accessIsFromLoongArch()) return;
   		if (notice.getAccessType() != AccessNotice.READ) return;
   		MemoryAccessNotice man = (MemoryAccessNotice) notice;
   		int currentAdress = man.getAddress();

   		if (currentAdress == lastAddress) return;
   		lastAddress = currentAdress;
   		ProgramStatement stmt;		
   		
   		try {
   			BasicInstruction instr = null;
   			stmt = Memory.getInstance().getStatement(currentAdress);
   			if(stmt == null){
   				return;
   			}
   			
   			instr = (BasicInstruction) stmt.getInstruction();
   			instructionBinary = stmt.getMachineStatement();
   			BasicInstructionFormat format = instr.getInstructionFormat();
   			painel.removeAll();
   			datapathAnimation = new DatapathAnimation(instructionBinary,instr);
   			this.createActionObjects();
   			toolbar= this.setUpToolBar();
   			painel.add(toolbar, BorderLayout.NORTH);
   			painel.add(datapathAnimation, BorderLayout.WEST);
   			datapathAnimation.startAnimation(instructionBinary );

   		} catch (AddressErrorException e) {
   			e.printStackTrace();
   		}
   		
   		
   	}

   	public void updateDisplay(){
   		this.repaint();
   	}
   	
   	//set the tool bar that controls the step in a time instruction running.
       private JToolBar setUpToolBar() {
           JToolBar toolBar = new JToolBar();
           Assemble = new JButton(runAssembleAction);
           Assemble.setText(""); 
           runBackStep = new JButton(runBackstepAction);
           runBackStep.setText(""); 
           
           Step = new JButton(runStepAction);
           Step.setText("");      	
           toolBar.add(Assemble);
           toolBar.add(Step);

           return toolBar;     
       }
       
       //set action in the menu bar.
       private void createActionObjects() {
           Toolkit tk = Toolkit.getDefaultToolkit();
           Class cs = this.getClass();
           try{
               runAssembleAction = new RunAssembleAction("Assemble",  
                       new ImageIcon(tk.getImage(cs.getResource(Globals.imagesPath+"Assemble22.png"))),
   							  "Assemble the current file and clear breakpoints", new Integer(KeyEvent.VK_A),
   							  KeyStroke.getKeyStroke( KeyEvent.VK_F3, 0), 
   							  mainUI);			

               runStepAction = new RunStepAction("Step", 
                       new ImageIcon(tk.getImage(cs.getResource(Globals.imagesPath+"StepForward22.png"))),
   							  "Run one step at a time", new Integer(KeyEvent.VK_T),
   							  KeyStroke.getKeyStroke( KeyEvent.VK_F7, 0),
   							  mainUI);	
               runBackstepAction = new RunBackstepAction("Backstep", 
                       new ImageIcon(tk.getImage(cs.getResource(Globals.imagesPath+"StepBack22.png"))),
   							  "Undo the last step", new Integer(KeyEvent.VK_B),
   							  KeyStroke.getKeyStroke( KeyEvent.VK_F8, 0), 
   							  mainUI);		
           }
           catch(Exception e){
               System.out.println("Internal Error: images folder not found, or other null pointer exception while creating Action objects");
               e.printStackTrace();
               System.exit(0);
           }
       }


class Vertex {
	private int numIndex;
   	private int init;
   	private int end;
   	private int current;
   	private String name;
   	public static final int movingUpside = 1;
   	public static final int movingDownside = 2;
   	public static final int movingLeft = 3;
   	public static final int movingRight = 4;
   	public int direction;
   	public int oppositeAxis;
   	private boolean isMovingXaxis;
   	private Color color;
   	private boolean first_interaction;
   	private boolean active;
   	private boolean isText;
   	private ArrayList<Integer> targetVertex;
   	
   	public Vertex(int index, int init, int end, String name, int oppositeAxis, boolean isMovingXaxis, 
   			String listOfColors, String listTargetVertex, boolean isText){
   		this.numIndex = index;
   		this.init = init;
   		this.current = this.init;
   		this.end = end; 
   		this.name = name;
   		this.oppositeAxis = oppositeAxis;
   		this.isMovingXaxis = isMovingXaxis;
   		this.first_interaction = true;
   		this.active = false;
   		this.isText = isText;
   		this.color = new Color(0,153,0);
   		if(isMovingXaxis == true){
   			if( init < end)
   				direction = movingLeft;
   			else 
   				direction = movingRight;
   			
   		}

   		else{
   			if( init < end)
   				direction = movingUpside;
   			else 
   				direction = movingDownside;
   		}
   		String[] list =  listTargetVertex.split("#");
   		targetVertex = new ArrayList<Integer>();
   		for(int i = 0; i < list.length; i++){
   			targetVertex.add(Integer.parseInt(list[i]));
   		//	System.out.println("Adding " + i + " " +  Integer.parseInt(list[i])+ " in target");
   		}
   		String[] listColor =  listOfColors.split("#");
   		this.color = new Color(Integer.parseInt(listColor[0]) , Integer.parseInt(listColor[1]),  Integer.parseInt(listColor[2]) );
   	}
 	
   	public int getDirection(){
   		return direction;
   	}
   	
	public boolean isText(){
   		return this.isText;
   	}


	public ArrayList<Integer> getTargetVertex() {
		return targetVertex;
	}

	public int getNumIndex() {
		return numIndex;
	}
	public void setNumIndex(int numIndex) {
		this.numIndex = numIndex;
	}
	public int getInit() {
		return init;
	}
	public void setInit(int init) {
		this.init = init;
	}
	public int getEnd() {
		return end;
	}
	public void setEnd(int end) {
		this.end = end;
	}
	public int getCurrent() {
		return current;
	}
	public void setCurrent(int current) {
		this.current = current;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getOppositeAxis() {
		return oppositeAxis;
	}
	public void setOppositeAxis(int oppositeAxis) {
		this.oppositeAxis = oppositeAxis;
	}
	public boolean isMovingXaxis() {
		return isMovingXaxis;
	}
	public void setMovingXaxis(boolean isMovingXaxis) {
		this.isMovingXaxis = isMovingXaxis;
	}
	public Color getColor() {
		return color;
	}
	public void setColor(Color color) {
		this.color = color;
	}
	public boolean isFirst_interaction() {
		return first_interaction;
	}
	public void setFirst_interaction(boolean first_interaction) {
		this.first_interaction = first_interaction;
	}
	public boolean isActive() {
		return active;
	}
	public void setActive(boolean active) {
		this.active = active;
	}
}


//Internal class that set the parameters value, control the basic behavior of the animation , and execute the animation of the 
//selected instruction in memory.
class DatapathAnimation extends JPanel
	implements ActionListener, MouseListener {
	/**
	 * 
	 */
	 private static final long serialVersionUID = -2681757800180958534L;

	 //config variables
	 private int PERIOD = 10;    // velocity of frames in ms
	 private static final int PWIDTH = 1102;     // size of this panel
	 private static final int PHEIGHT = 1162;
	 private GraphicsConfiguration gc;
	 private GraphicsDevice gd;   	// for reporting accl. memory usage
	 private int accelMemory;   
	 private DecimalFormat df;
	 
	 private int counter;			//verify then remove.
	 private boolean justStarted; 	//flag to start movement


	 private int indexX;	//counter of screen position
	 private int indexY;
	 private boolean xIsMoving, yIsMoving; 		//flag for mouse movement.
	 	 
	 

//	 private Vertex[][] inputGraph;
	 private Vector<Vector<Vertex>>  outputGraph;
	 private ArrayList<Vertex> vertexList;
	 private ArrayList<Vertex> vertexTraversed;
	 //Screen Label variables
	 
	 private HashMap<String, String> opcodeEquivalenceTable;
	 private HashMap<String, String> functionEquivalenceTable;
	 private HashMap<String, String> registerEquivalenceTable;

	 private String instructionCode;
	 private BasicInstruction instruction;
	 private int countRegLabel;
	 private int countALULabel;
	 private int countPCLabel;
	 
	 //Colors variables
	 private Color green1 = new Color(0,153,0);
	 private Color green2 = new Color( 0,77,0);
	 private Color yellow2 = new Color(185,182,42);
	 private Color orange1 = new Color(255,102,0);
	 private Color orange = new Color(119,34,34);
	 private Color blue2 = new Color(0,153,255);
	 
	 private int register = 1;
	 private int control = 2;
	 private int aluControl = 3;
	 private int alu = 4;
	 private int currentUnit;
	 private Graphics2D g2d;
	 
	 
	 private BufferedImage datapath;

	 public void mousePressed(MouseEvent e) {
		 	PointerInfo a = MouseInfo.getPointerInfo();
		 //	System.out.println("olha, capturado x=" + a.getLocation().getX() + " y = " + a.getLocation().getY());
	    }
	 
	 public DatapathAnimation(String instructionBinary,BasicInstruction instr)
	 {
		 df = new DecimalFormat("0.0");  // 1 dp
		 GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		 gd = ge.getDefaultScreenDevice();
		 gc = ge.getDefaultScreenDevice().getDefaultConfiguration();

		 accelMemory = gd.getAvailableAcceleratedMemory();  // in bytes
		 setBackground(Color.white);
		 setPreferredSize( new Dimension(PWIDTH, PHEIGHT) );

		 // load and initialise the images
		 initImages();
		  
		 vertexList = new ArrayList<Vertex>();
		 counter = 0;
		 justStarted = true;
		 instructionCode = instructionBinary;
		 instruction=instr;
		 
		 //declaration of labels definition.
		 opcodeEquivalenceTable = new  HashMap<String, String>();
		 functionEquivalenceTable = new  HashMap<String, String>();
		 registerEquivalenceTable = new  HashMap<String, String>();

		 countRegLabel = 400;
		 countALULabel = 380;
		 countPCLabel = 380;
		 loadHashMapValues();
		 addMouseListener(this);

		 

	 } // end of ImagesTests()


	//机器码 和 指令名称对应的的位置

	//先读取指令 -> 读取32位字码 -> 根据字码获取字符 ->从哈希表中获取
	//哈希表从xml文件中进行建立
	//那就ok拉

	//更改的话 就从划分开始改
	//对不同的指令有着不一样的划分
	//判断动画的过程？ 看动画模拟是怎么实现的？


	 //set the binnary opcode value of the basic instructions of LoongArch instruction set
	 public void loadHashMapValues(){
		 importXmlStringData("/LoongArchXRayOpcode.xml",opcodeEquivalenceTable, "equivalence",  "bits", "mnemonic");
		 importXmlStringData("/LoongArchXRayOpcode.xml", functionEquivalenceTable, "function_equivalence",  "bits", "mnemonic");
		 importXmlStringData("/LoongArchXRayOpcode(new).xml", registerEquivalenceTable, "register_equivalence",  "bits", "mnemonic");
	 	 importXmlDatapathMap("/LoongArchXRayOpcode(new).xml", "datapath_map");
	 }
	 
	 //import the list of opcodes of loongarch set of instructions
	 public void importXmlStringData(String xmlName, HashMap table, String elementTree, String tagId, String tagData){
		 	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(false);
			DocumentBuilder docBuilder;
			try {
				//System.out.println();
				docBuilder = dbf.newDocumentBuilder();
				Document doc = docBuilder.parse(getClass().getResource(xmlName).toString());
				Element root = doc.getDocumentElement();
				Element equivalenceItem;
				NodeList bitsList,  mnemonic;
				NodeList equivalenceList = root.getElementsByTagName(elementTree);
				for(int i = 0; i < equivalenceList.getLength(); i++){
					equivalenceItem =  (Element)equivalenceList.item(i);
					bitsList = equivalenceItem.getElementsByTagName(tagId);
					mnemonic = equivalenceItem.getElementsByTagName(tagData);
					for(int j= 0; j < bitsList.getLength(); j++){
						table.put(bitsList.item(j).getTextContent(),mnemonic.item(j).getTextContent());
					}
				}
			}
			 catch (Exception e) {
					e.printStackTrace();
				}
	 }

	 //LoongArch
	 public void importXmlDatapathMap(String xmlName, String elementTree){
		 DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		 dbf.setNamespaceAware(false);
		 DocumentBuilder docBuilder;
		 try {
			 docBuilder = dbf.newDocumentBuilder();
			 Document doc = docBuilder.parse(getClass().getResource(xmlName).toString());
			 Element root = doc.getDocumentElement();
			 Element datapath_mapItem;
			 NodeList index_vertex,  name, init, end,color, other_axis, isMovingXaxis, targetVertex, sourceVertex, isText ;
			 NodeList datapath_mapList = root.getElementsByTagName(elementTree);
			 //此处将datapath_map都读进去了
			 for(int i = 0; i < datapath_mapList.getLength(); i++){ //extract the vertex of the xml input and encapsulate into the vertex object
				 datapath_mapItem =  (Element)datapath_mapList.item(i);
				 index_vertex = datapath_mapItem.getElementsByTagName("num_vertex");
				 //这里实际上就是将里面都写入了

				 name = datapath_mapItem.getElementsByTagName("name");
				 init = datapath_mapItem.getElementsByTagName("init");
				 end = datapath_mapItem.getElementsByTagName("end");
				 //definition of colors line
				 /**
				  * 加入判断 如果verNum在某个范围就让他控制颜色根据那20种来即可
				  *
				  */

				 /**
				  * R type
				  * 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0->0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 1
				  *
				  * I12 type
				  *
				  * 运算类
				  * 0 0 0 0 0 0 1 0 0 0->0 0 0 0 0 0 1 1 1 1
				  *
				  *
				  * LOAD
				  *	0 0 1 0 1 0 0 0 0 0->0 0 1 0 1 0 0 0 1 0
				  *
				  *	0 0 1 0 1 0 1 0 0 0->0 0 1 0 1 0 1 0 0 1
				  *
				  * STORE
				  *0010100100->0 0 1 0 1 0 0 1 1 0
				  *
				  *
				  *
				  * J type
				  * 0 1 0 1 0 0 -> 0 1 0 1 0 1
				  *
				  * B type
				  * 0 1 0 0 1 1
				  * 0 1 0 1 1 0->0 1 1 0 1 1
				  */
				 var format= instruction.getInstructionFormat();
				 //非control总线的判断
				 if(format==(BasicInstructionFormat.TWO_R_TYPE)||
						 format==(BasicInstructionFormat.THREE_R_TYPE)||
						 format==(BasicInstructionFormat.FOUR_R_TYPE)){
					 color=datapath_mapItem.getElementsByTagName("color_Rtype");
					 //System.out.println("rType");
				 }
				 else if(format==(BasicInstructionFormat.TWO_R_I12_TYPE)){
					 if(instructionCode.substring(0,10).matches("000000[0-1][0-1][0-1][0-1]")){
						 color=datapath_mapItem.getElementsByTagName("color_Itype");
					 }
					 else if (instructionCode.substring(0,10).matches("0010100000")||
							 instructionCode.substring(0,10).matches("0010100001")||
							 instructionCode.substring(0,10).matches("0010100010")||
							 instructionCode.substring(0,10).matches("001010100[0-1]")) {
						 color=datapath_mapItem.getElementsByTagName("color_LOADtype");
					 }
					 else if(instructionCode.substring(0,10).matches("0010100100")||
							 instructionCode.substring(0,10).matches("0010100101")||
							 instructionCode.substring(0,10).matches("0010100110")){
						 color=datapath_mapItem.getElementsByTagName("color_STOREtype");
					 }
					 else
						 color=datapath_mapItem.getElementsByTagName("color_Itype");
				 }
				 else if(format==BasicInstructionFormat.ONE_R_I20_TYPE){
					 color=datapath_mapItem.getElementsByTagName("color_ONE_R_I20_type");
				 }
				 else if(format==(BasicInstructionFormat.JUMP_TYPE)){
					 color=datapath_mapItem.getElementsByTagName("color_Jtype");
				 }
				 else if(format==BasicInstructionFormat.I26_TYPE){
					 if(instructionCode.substring(0,6).matches("010100"))
						 color=datapath_mapItem.getElementsByTagName("color_B");
					 else if(instructionCode.substring(0,6).matches("010101"))
						 color=datapath_mapItem.getElementsByTagName("color_BL");
					 else
						 color=datapath_mapItem.getElementsByTagName("color_Itype");
				 }
				 else if (format==BasicInstructionFormat.TWO_R_I16_BRANCH_TYPE){
					 color=datapath_mapItem.getElementsByTagName("color_BRANCHtype");
				 }
				 else{
					 color=datapath_mapItem.getElementsByTagName("color_Itype");
				 }

				 other_axis = datapath_mapItem.getElementsByTagName("other_axis");
				 isMovingXaxis = datapath_mapItem.getElementsByTagName("isMovingXaxis");
				 targetVertex = datapath_mapItem.getElementsByTagName("target_vertex");
				 isText = datapath_mapItem.getElementsByTagName("is_text");


				 for(int j= 0; j < index_vertex.getLength(); j++){
					 Vertex vert = new Vertex(
							 Integer.parseInt(index_vertex.item(j).getTextContent()),
							 Integer.parseInt(init.item(j).getTextContent()),
							 Integer.parseInt(end.item(j).getTextContent()),
							 name.item(j).getTextContent(),
							 Integer.parseInt(other_axis.item(j).getTextContent()),
							 Boolean.parseBoolean(isMovingXaxis.item(j).getTextContent()),
							 color.item(j).getTextContent(),
							 targetVertex.item(j).getTextContent(),
							 Boolean.parseBoolean(isText.item(j).getTextContent()));
					 vertexList.add(vert);
				 }
			 }
			 //loading matrix of control of vertex.
			 outputGraph = new Vector<Vector<Vertex>>();
			 vertexTraversed = new ArrayList<Vertex>();
			 int size = vertexList.size();
			 Vertex vertex;
			 ArrayList<Integer> targetList;
			 for(int i = 0; i < vertexList.size(); i++){
				 vertex = vertexList.get(i);
				 targetList = vertex.getTargetVertex();
				 Vector<Vertex> vertexOfTargets = new Vector<Vertex>();
				 for(int k = 0; k < targetList.size(); k++){
					 vertexOfTargets.add(vertexList.get(targetList.get(k)));
				 }
				 outputGraph.add(vertexOfTargets);
			 }
			 for(int i=0; i< outputGraph.size(); i++){
				 Vector<Vertex> vert = outputGraph.get(i);
			 }
			 vertexList.get(0).setActive(true);
			 vertexTraversed.add(vertexList.get(0));
		 }
		 catch (Exception e) {
			 e.printStackTrace();
		 }

	 }

	//Set up the information showed in the screen of the current instruction.
	 public void setUpInstructionInfo( Graphics2D g2d) {
		 //根据前6位去读取
		 //
		 FontRenderContext frc = g2d.getFontRenderContext();
		 Font font = new Font("Digital-7", Font.PLAIN, 15);
		 Font fontTitle = new Font("Verdana", Font.PLAIN, 10);

		 TextLayout textVariable;
		 var format = instruction.getInstructionFormat();
		 /**
		  * 3R- 后15位 31-15=16 0->17 17->22 22->27 27->32
		  */


		 //获取指令助记符
		 String instrName = instruction.getExampleFormat().split(" ")[0];
		 if (format == BasicInstructionFormat.THREE_R_TYPE) {


			 textVariable = new TextLayout("REGISTER TYPE INSTRUCTION", new Font("Arial", Font.BOLD, 15), frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 20, 50);
			 //opcode label
			 textVariable = new TextLayout("opcode", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 160);

			 //initialize of opcode
			 textVariable = new TextLayout(instructionCode.substring(0, 17), font, frc);
			 g2d.setColor(Color.magenta);
			 textVariable.draw(g2d, 25, 190);

			 //rs label
			 textVariable = new TextLayout("rk", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 220);

			 //initialize of rs
			 textVariable = new TextLayout(instructionCode.substring(17, 22), font, frc);
			 g2d.setColor(Color.green);
			 textVariable.draw(g2d, 25, 250);

			 //rt label
			 textVariable = new TextLayout("rj", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 85, 220);

			 //initialize of rt
			 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
			 g2d.setColor(Color.blue);
			 textVariable.draw(g2d, 85, 250);

			 // rd label
			 textVariable = new TextLayout("rd", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 145, 220);

			 //initialize of rd
			 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 145, 250);


			 //instruction mnemonic
			 textVariable = new TextLayout("Instruction", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 110);

			 //instruction name
			 textVariable = new TextLayout(instrName + "    ", font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 25, 130);

			 //register in RS
			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(17, 22)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 75, 130);

			 //register in RT
			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(22, 27)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 115, 130);

			 //register in RD
			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 155, 130);
		 } else if (format == BasicInstructionFormat.TWO_R_I12_TYPE) {
			 //2R-I12 运算指令
			 if (instructionCode.substring(0, 10).matches("000000[0-1][0-1][0-1][0-1]")) {

				 textVariable = new TextLayout("IMMEDIATE TYPE INSTRUCTION", new Font("Arial", Font.BOLD, 15), frc);
				 g2d.setColor(Color.black);
				 textVariable.draw(g2d, 20, 50);
				 //opcode label
				 textVariable = new TextLayout("opcode", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 160);

				 //initialize of opcode
				 textVariable = new TextLayout(instructionCode.substring(0, 10), font, frc);
				 g2d.setColor(Color.magenta);
				 textVariable.draw(g2d, 25, 190);

				 //immediate label
				 textVariable = new TextLayout("immediate", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 220);

				 //initialize of rs
				 textVariable = new TextLayout(instructionCode.substring(10, 22), font, frc);
				 g2d.setColor(Color.green);
				 textVariable.draw(g2d, 25, 250);

				 //rj label
				 textVariable = new TextLayout("rj", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 280);

				 //initialize of rj
				 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
				 g2d.setColor(Color.blue);
				 textVariable.draw(g2d, 25, 310);

				 // rd label
				 textVariable = new TextLayout("rd", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 85, 280);

				 //initialize of rd
				 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
				 g2d.setColor(Color.cyan);
				 textVariable.draw(g2d, 85, 310);


				 //instruction mnemonic
				 textVariable = new TextLayout("Instruction", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 110);

				 //instruction name
				 textVariable = new TextLayout(instrName + " ", font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 25, 130);

				 //register in IMM
				 textVariable = new TextLayout(parseBinToInt(instructionCode.substring(10, 22)), font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 75, 130);

				 //register in RJ
				 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(22, 27)), font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 115, 130);

				 //register in RD
				 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 155, 130);
			 }

			 //Load instruction

			 else if (instructionCode.substring(0, 10).matches("0010100000") ||
					 instructionCode.substring(0, 10).matches("0010100001") ||
					 instructionCode.substring(0, 10).matches("0010100010") ||
					 instructionCode.substring(0, 10).matches("001010100[0-1]")) {

				 textVariable = new TextLayout("LOAD INSTRUCTION", new Font("Arial", Font.BOLD, 15), frc);
				 g2d.setColor(Color.black);
				 textVariable.draw(g2d, 20, 50);
				 //opcode label
				 textVariable = new TextLayout("opcode", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 160);

				 //initialize of opcode
				 textVariable = new TextLayout(instructionCode.substring(0, 10), font, frc);
				 g2d.setColor(Color.magenta);
				 textVariable.draw(g2d, 25, 190);

				 //immediate label
				 textVariable = new TextLayout("immediate", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 220);

				 //initialize of imm
				 textVariable = new TextLayout(instructionCode.substring(10, 22), font, frc);
				 g2d.setColor(Color.green);
				 textVariable.draw(g2d, 25, 250);

				 //rj label
				 textVariable = new TextLayout("rj", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 280);

				 //initialize of rj
				 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
				 g2d.setColor(Color.blue);
				 textVariable.draw(g2d, 25, 310);

				 // rd label
				 textVariable = new TextLayout("rd", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 85, 280);

				 //initialize of rd
				 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
				 g2d.setColor(Color.cyan);
				 textVariable.draw(g2d, 85, 310);

				 //instruction mnemonic
				 textVariable = new TextLayout("Instruction", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 110);

				 //instruction name
				 textVariable = new TextLayout(instrName + " ", font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 25, 130);

				 //register in RD
				 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 75, 130);

				 //Load Target Memory
				 textVariable = new TextLayout("M[ " + registerEquivalenceTable.get(instructionCode.substring(22, 27)) + " + " + parseBinToInt(instructionCode.substring(10, 22)) + " ]", font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 105, 130);

			 }

			 //STORE INSTRUCTION
			 else if (instructionCode.substring(0, 10).matches("0010100100") ||
					 instructionCode.substring(0, 10).matches("0010100101") ||
					 instructionCode.substring(0, 10).matches("0010100110")) {

				 textVariable = new TextLayout("STORE INSTRUCTION", new Font("Arial", Font.BOLD, 15), frc);
				 g2d.setColor(Color.black);
				 textVariable.draw(g2d, 20, 50);
				 //opcode label
				 textVariable = new TextLayout("opcode", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 160);

				 //initialize of opcode
				 textVariable = new TextLayout(instructionCode.substring(0, 10), font, frc);
				 g2d.setColor(Color.magenta);
				 textVariable.draw(g2d, 25, 190);

				 //immediate label
				 textVariable = new TextLayout("immediate", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 220);

				 //initialize of imm
				 textVariable = new TextLayout(instructionCode.substring(10, 22), font, frc);
				 g2d.setColor(Color.green);
				 textVariable.draw(g2d, 25, 250);

				 //rj label
				 textVariable = new TextLayout("rj", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 280);

				 //initialize of rj
				 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
				 g2d.setColor(Color.blue);
				 textVariable.draw(g2d, 25, 310);

				 // rd label
				 textVariable = new TextLayout("rd", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 85, 280);

				 //initialize of rd
				 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
				 g2d.setColor(Color.cyan);
				 textVariable.draw(g2d, 85, 310);


				 //instruction mnemonic
				 textVariable = new TextLayout("Instruction", fontTitle, frc);
				 g2d.setColor(Color.red);
				 textVariable.draw(g2d, 25, 110);

				 //instruction name
				 textVariable = new TextLayout(instrName + " ", font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 25, 130);


				 //register in RD
				 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 75, 130);

				 //Store Target Memory
				 textVariable = new TextLayout("M[ " + registerEquivalenceTable.get(instructionCode.substring(22, 27)) + " + " + parseBinToInt(instructionCode.substring(10, 22)) + " ]", font, frc);
				 g2d.setColor(Color.BLACK);
				 textVariable.draw(g2d, 115, 130);


			 }
		 }
		 //jump instruction
		 else if (format == BasicInstructionFormat.JUMP_TYPE) {
			 textVariable = new TextLayout("JUMP TYPE INSTRUCTION", new Font("Verdana", Font.BOLD, 15), frc); //description of instruction code type for jump.
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 20, 50);

			 // label opcode
			 textVariable = new TextLayout("opcode", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 160);

			 //initialize of opcode
			 textVariable = new TextLayout(instructionCode.substring(0, 6), font, frc);
			 g2d.setColor(Color.magenta);
			 textVariable.draw(g2d, 25, 190);

			 //immediate label
			 textVariable = new TextLayout("offset", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 220);

			 //initialize of rj
			 textVariable = new TextLayout(instructionCode.substring(6, 22), font, frc);
			 g2d.setColor(Color.green);
			 textVariable.draw(g2d, 25, 250);

			 //rj label
			 textVariable = new TextLayout("rj", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 280);

			 //initialize of rj
			 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
			 g2d.setColor(Color.blue);
			 textVariable.draw(g2d, 25, 310);

			 // rd label
			 textVariable = new TextLayout("rd", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 85, 280);

			 //initialize of rd
			 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 85, 310);


			 //instruction mnemonic
			 textVariable = new TextLayout("Instruction", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 110);

			 //instruction name
			 textVariable = new TextLayout(instrName + " ", font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 25, 130);

			 //register in IMM
			 textVariable = new TextLayout(parseBinToInt(instructionCode.substring(6, 22)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 75, 130);

			 //register in RJ
			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(22, 27)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 115, 130);

			 //register in RD
			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
			 g2d.setColor(Color.BLACK);
			 textVariable.draw(g2d, 155, 130);
		 }

		 //直接跳转指令B BL
		 else if (instructionCode.substring(0, 6).matches("01010[0-1]")) {
			 textVariable = new TextLayout("BRANCH UNCONDITIONAL TYPE INSTRUCTION", new Font("Verdana", Font.BOLD, 15), frc); //description of instruction code type for jump.
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 20, 50);

			 // label opcode
			 textVariable = new TextLayout("opcode", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 160);

			 //initialize of opcode
			 textVariable = new TextLayout(instructionCode.substring(0, 6), font, frc);
			 g2d.setColor(Color.magenta);
			 textVariable.draw(g2d, 25, 190);

			 //label address
			 textVariable = new TextLayout("offset", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 220);

			 //initialize of adress
			 textVariable = new TextLayout(instructionCode.substring(6, 32), font, frc);
			 g2d.setColor(Color.orange);
			 textVariable.draw(g2d, 25, 250);

			 textVariable = new TextLayout("Instruction", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 110);

			 //instruction mnemonic
			 textVariable = new TextLayout(instrName + " ", font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 25, 130);

			 //instruction immediate
			 textVariable = new TextLayout("LABEL", font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 75, 130);
		 }

		 //有条件跳转指令
		 else if (format == BasicInstructionFormat.TWO_R_I16_BRANCH_TYPE) {
			 textVariable = new TextLayout("BRANCH TYPE INSTRUCTION", new Font("Verdana", Font.BOLD, 15), frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 20, 50);

			 //label opcode
			 textVariable = new TextLayout("opcode", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 160);

			 //initialize of opcode
			 textVariable = new TextLayout(instructionCode.substring(0, 6), font, frc);
			 g2d.setColor(Color.magenta);
			 textVariable.draw(g2d, 25, 190);

			 //imm label
			 textVariable = new TextLayout("Immediate", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 220);
			 //initialize of immediate
			 textVariable = new TextLayout(instructionCode.substring(6, 22), font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 25, 250);

			 //rj label
			 textVariable = new TextLayout("rj", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 280);

			 //initialize of rj
			 textVariable = new TextLayout(instructionCode.substring(22, 27), font, frc);
			 g2d.setColor(Color.blue);
			 textVariable.draw(g2d, 25, 310);

			 // rd label
			 textVariable = new TextLayout("rd", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 85, 280);

			 //initialize of rd
			 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 85, 310);

			 //instruction mnemonic
			 textVariable = new TextLayout("Instruction", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 110);

			 textVariable = new TextLayout(instrName + " ", font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 25, 130);

			 textVariable = new TextLayout(parseBinToInt(instructionCode.substring(6, 22)), font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 75, 130);

			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(22, 27)), font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 115, 130);

			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 155, 130);


		 }
		 else if(format==BasicInstructionFormat.ONE_R_I20_TYPE){
			 textVariable = new TextLayout("ONR_R_I20 TYPE INSTRUCTION", new Font("Verdana", Font.BOLD, 15), frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 20, 50);

			 //label opcode
			 textVariable = new TextLayout("opcode", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 160);

			 //initialize of opcode
			 textVariable = new TextLayout(instructionCode.substring(0, 7), font, frc);
			 g2d.setColor(Color.magenta);
			 textVariable.draw(g2d, 25, 190);

			 //imm label
			 textVariable = new TextLayout("Immediate", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 220);
			 //initialize of immediate
			 textVariable = new TextLayout(instructionCode.substring(7, 27), font, frc);
			 g2d.setColor(Color.cyan);
			 textVariable.draw(g2d, 25, 250);

			 //rd label
			 textVariable = new TextLayout("rd", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 280);

			 //initialize of rd
			 textVariable = new TextLayout(instructionCode.substring(27, 32), font, frc);
			 g2d.setColor(Color.blue);
			 textVariable.draw(g2d, 25, 310);

			 //instruction mnemonic
			 textVariable = new TextLayout("Instruction", fontTitle, frc);
			 g2d.setColor(Color.red);
			 textVariable.draw(g2d, 25, 110);

			 textVariable = new TextLayout(instrName + " ", font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 25, 130);

			 textVariable = new TextLayout(parseBinToInt(instructionCode.substring(7, 27)), font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 75, 130);

			 textVariable = new TextLayout(registerEquivalenceTable.get(instructionCode.substring(27, 32)), font, frc);
			 g2d.setColor(Color.black);
			 textVariable.draw(g2d, 145, 130);
		 }
		 //Type of control signal labels
		 textVariable = new TextLayout("Control Signals", fontTitle, frc);
		 g2d.setColor(Color.red);
		 textVariable.draw(g2d, 25, 70);

		 textVariable = new TextLayout("Active", font, frc);
		 g2d.setColor(Color.red);
		 textVariable.draw(g2d, 25, 85);

		 textVariable = new TextLayout("Inactive", font, frc);
		 g2d.setColor(Color.gray);
		 textVariable.draw(g2d, 75, 85);
	 }
	 //end of instruction subtitle...
	 
	 
	//set the initial state of the variables that controls the animation, and start the timer that triggers the animation. 
	 public void startAnimation(String codeInstruction){
	 	instructionCode = codeInstruction;
	 	time = new Timer(PERIOD, this);    // start timer
	 	time.start();
	// 	this.repaint();
	 }
	 
	 //initialize the image of datapath.
	 private void initImages(){
		 try {
			 BufferedImage im =  ImageIO.read( 
					 //getClass().getResource(Globals.imagesPath+"datapath.png") );
					 getClass().getResource(Globals.imagesPath+"DatapathLA.png") );
			 int transparency = im.getColorModel().getTransparency();
			 datapath =  gc.createCompatibleImage(
					 im.getWidth(), im.getHeight(),
					 transparency );
			 g2d = datapath.createGraphics();


			 g2d.drawImage(im,0,0,null);
			 g2d.dispose();
		 } 
		 catch(IOException e) {
			 System.out.println("Load Image error for " +
					 //getClass().getResource(Globals.imagesPath+"datapath.png") + ":\n" + e);
					 getClass().getResource(Globals.imagesPath+"DatapathLA.png") + ":\n" + e);
		 }
	 } 

	 
	 public void actionPerformed(ActionEvent e)
	 // triggered by the timer: update, repaint
	 { 
		 if (justStarted)   
			 justStarted = false;
		 if(xIsMoving)
			 indexX++;
		 if(yIsMoving)
			 indexY--;
		 repaint(); 
	 } 


	 public void paintComponent(Graphics g)
	 {
		 super.paintComponent(g);
		 g2d = (Graphics2D)g; 
		 // use antialiasing
		 g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				 RenderingHints.VALUE_ANTIALIAS_ON);
		 // smoother (and slower) image transformations  (e.g. for resizing)
		 g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				 RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		 g2d = (Graphics2D)g;
		 drawImage(g2d, datapath, 0,0,null);
		 executeAnimation(g);
		 counter = (counter + 1)% 100;
		 g2d.dispose();
		 
	 } 

	 private void drawImage(Graphics2D g2d, BufferedImage im, int x, int y,Color c){ 
		 if (im == null) {
			 g2d.setColor(c);
			 g2d.fillOval(x, y, 20, 20);
			 g2d.setColor(Color.black);
			 g2d.drawString("   ", x, y);
		 }
		 else
			 g2d.drawImage(im, x, y, this);
	 } 

	//draw lines.
	 //method to draw the lines that run from left to right.
	 public void printTrackLtoR(Vertex v){
		 int size;
		 int[] track;
		 size = v.getEnd() - v.getInit();
		 track =  new int[size];
		 for(int i = 0; i < size; i++)
			 track[i] = v.getInit()+i;
		 if(v.isActive() == true){
			 	v.setFirst_interaction(false);
				for(int i = 0; i < size; i++){
					if(track[i] <= v.getCurrent()){
						g2d.setColor(v.getColor());  
						g2d.fillRect(track[i], v.getOppositeAxis(), 3, 3);
					}
				}	
				if (v.getCurrent() == track[size-1])
					v.setActive(false);
				v.setCurrent(v.getCurrent()+1);	
		 }
		  else if(v.isFirst_interaction() == false){
				 for(int i = 0; i < size ; i++){
					 g2d.setColor(v.getColor());  
					 g2d.fillRect(track[i],v.getOppositeAxis(),  3, 3);
				 }	
		 }
		
	 }

	 //method to draw the lines that run from right to left.
	 //public boolean printTrackRtoL(int init, int end ,int currentIndex, Graphics2D g2d, Color color, int otherAxis, 
	//		 boolean active, boolean firstInteraction){
	public void printTrackRtoL(Vertex v){
		int size;
		int[] track;
		size = v.getInit() - v.getEnd();
		track =  new int[size];
		
		for(int i = 0; i < size; i++)
			track[i] = v.getInit()-i; 

		if(v.isActive() == true){
			v.setFirst_interaction(false);
			for(int i = 0; i < size; i++){
				if(track[i] >= v.getCurrent()){
					g2d.setColor(v.getColor());  
					g2d.fillRect(track[i], v.getOppositeAxis(), 3, 3);
				}
			}
			if (v.getCurrent() == track[size-1])
				v.setActive(false);
			
			v.setCurrent(v.getCurrent()-1);	
		 }
		 else if(v.isFirst_interaction() == false){
				for(int i = 0; i < size ; i++){
					g2d.setColor(v.getColor());  
					g2d.fillRect(track[i],v.getOppositeAxis(),  3, 3);
				 }
		 }
	 }

	 //method to draw the lines that run from down to top.
	// public boolean printTrackDtoU(int init, int end ,int currentIndex, Graphics2D g2d, Color color, int otherAxis, 
	//		 boolean active, boolean firstInteraction){
	 public void printTrackDtoU(Vertex v){	 
		 int size;
		 int[] track;

		 if(v.getInit() > v.getEnd()){
			 size = v.getInit() - v.getEnd();
			 track =  new int[size];
			 for(int i = 0; i < size; i++)
				 track[i] = v.getInit()-i;
		 }
		 else{
			 size = v.getEnd() - v.getInit();
			 track =  new int[size];
			 for(int i = 0; i < size; i++)
				 track[i] = v.getInit()+i;
		 }

		if(v.isActive() == true){
			 v.setFirst_interaction(false);
			 for(int i = 0; i < size; i++){
				 if(track[i] >= v.getCurrent()){
					 g2d.setColor(v.getColor());  
					 g2d.fillRect(v.getOppositeAxis(), track[i], 3, 3);	
				 }
			 }
			 if (v.getCurrent() == track[size-1])
				 v.setActive(false);
			 v.setCurrent(v.getCurrent()-1);	

		 }
		  else if(v.isFirst_interaction() == false){
				 for(int i = 0; i < size; i++){
					 g2d.setColor(v.getColor());  
					 g2d.fillRect(v.getOppositeAxis(), track[i], 3, 3);
				 }
		 }
	 }
	 //method to draw the lines that run from top to down.
	// public boolean printTrackUtoD(int init, int end ,int currentIndex, Graphics2D g2d, Color color, int otherAxis, 
	//		 boolean active,  boolean firstInteraction){
	 public void printTrackUtoD(Vertex v){
		 	 
		 int size;
		 int[] track;
		 size = v.getEnd() - v.getInit();
		 track =  new int[size];
		 
		 for(int i = 0; i < size; i++)
			 track[i] = v.getInit()+i;

		 if(v.isActive() == true){
			 v.setFirst_interaction(false);
			 for(int i = 0; i < size; i++){
				 if(track[i] <= v.getCurrent()){
					 g2d.setColor(v.getColor());  
					 g2d.fillRect(v.getOppositeAxis(), track[i], 3, 3);	
				 }
				 	
			 }
			 if (v.getCurrent() == track[size-1]) 
				 v.setActive(false);
			 v.setCurrent(v.getCurrent()+1);
		 }
		  else if(v.isFirst_interaction() == false){
				 for(int i = 0; i < size; i++){
					 g2d.setColor(v.getColor());  
					 g2d.fillRect(v.getOppositeAxis(), track[i], 3, 3);
				 }
		 }
	 }
	
	 public void printTextDtoU(Vertex v){	 
		 int size;
		 int[] track;
		 FontRenderContext frc = g2d.getFontRenderContext();
		 
		 TextLayout actionInFunctionalBlock = new TextLayout(v.getName(), new Font("Verdana", Font.BOLD, 13), frc);
		 g2d.setColor(Color.RED);
/*		 if(instructionCode.substring(0,6).matches("101[0-1][0-1][0-1]")//?浮点数指令?
				 &&!instructionCode.substring(0,6).matches("0001[0-1][0-1]")//branch
				 	&&!instructionCode.substring(0,6).matches("00001[0-1]")){//load instruction
			 actionInFunctionalBlock = new TextLayout(" ", new Font("Verdana", Font.BOLD, 13), frc);
		 }*/

		 //对ALU的指令选择
		 if(v.getName().equals("ALU_VALUE")){
			 //op_add
			 if(instructionCode.substring(0,17).matches("00000000000100000")||//add.w
			 	instructionCode.substring(0,10).matches("0000001010")||//addi.w
			 	(instructionCode.substring(0,10).matches("0010100000")||//load instruction
				 instructionCode.substring(0,10).matches("0010100001")||
				 instructionCode.substring(0,10).matches("0010100010")||
				 instructionCode.substring(0,10).matches("001010100[0-1]"))||
			    (instructionCode.substring(0,10).matches("0010100100")||
				 instructionCode.substring(0,10).matches("0010100101")||
				 instructionCode.substring(0,10).matches("0010100110"))||//store instruction
				 instructionCode.substring(0,6).matches("010011")||//JIRL
				 instructionCode.substring(0,6).matches("010101")
			 	){
				 actionInFunctionalBlock = new TextLayout("op_add", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_sub
			 else if(instructionCode.substring(0,17).matches("00000000000100010")){//sub.w
				 actionInFunctionalBlock = new TextLayout("op_sub", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_slt
			 else if(instructionCode.substring(0,17).matches("00000000000100100")||//slt
			 	instructionCode.substring(0,10).matches("0000001000")){//slti
				 actionInFunctionalBlock = new TextLayout("op_slt", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_sltu
			 else if(instructionCode.substring(0,17).matches("00000000000100101")||//sltu
			    instructionCode.substring(0,10).matches("0000001001")){//sltui
				 actionInFunctionalBlock = new TextLayout("op_sltu", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_and
			 else if(instructionCode.substring(0,17).matches("00000000000101001")||//and
			 	instructionCode.substring(0,10).matches("0000001101")){//andi
				 actionInFunctionalBlock = new TextLayout("op_and", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_nor
			 else if(instructionCode.substring(0,17).matches("00000000000101000")){//nor
				 actionInFunctionalBlock = new TextLayout("op_nor", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_or
			 else if(instructionCode.substring(0,17).matches("00000000000101010")||//or
			 	instructionCode.substring(0,10).matches("0000001110")){//ori
				 actionInFunctionalBlock = new TextLayout("op_or", new Font("Verdana", Font.BOLD, 13), frc);

			 }
			 //op_xor
			 else if(instructionCode.substring(0,17).matches("00000000000101011")||//xor
					 instructionCode.substring(0,10).matches("0000001111")){//xori
				 actionInFunctionalBlock = new TextLayout("op_xor", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_sll
			 else if(instructionCode.substring(0,17).matches("00000000000101110")||//sll.w
					 instructionCode.substring(0,14).matches("00000000010000")){//slli.w
				 actionInFunctionalBlock = new TextLayout("op_sll", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_srl
			 else if(instructionCode.substring(0,17).matches("00000000000101111")||//srl.w
					 instructionCode.substring(0,14).matches("00000000010001")){//srli.w
				 actionInFunctionalBlock = new TextLayout("op_srl", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_sra
			 else if(instructionCode.substring(0,17).matches("00000000000110000")||//sra.w
					 instructionCode.substring(0,14).matches("00000000010010")){//srai.w
				 actionInFunctionalBlock = new TextLayout("op_sra", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //op_lui
			 else if(instructionCode.substring(0,7).matches("0001010")){
				 actionInFunctionalBlock = new TextLayout("op_lui", new Font("Verdana", Font.BOLD, 13), frc);
			 }
			 //nop
			 else
				 actionInFunctionalBlock = new TextLayout(" ", new Font("Verdana", Font.BOLD, 13), frc);
		 }
		 //跳转指令
		 if((instructionCode.substring(0,6).matches("010011")||
			 instructionCode.substring(0,6).matches("0101[0-1][0-1]")||
			 instructionCode.substring(0,6).matches("0110[0-1][0-1]"))
			 && v.getName().equals("PC+4")) //branch code
			 actionInFunctionalBlock = new TextLayout("PC+OFFSET", new Font("Verdana", Font.BOLD, 13), frc);

		 //LOAD指令
		 //0 0 1 0 1 0 0 0 0 0
		 //0010100010
		 if(v.getName().equals("WRITING")){
			 //不是load类指令的就不能writing
			 if(!(instructionCode.substring(0,10).matches("0010100000")||
					 instructionCode.substring(0,10).matches("0010100001")||
					 instructionCode.substring(0,10).matches("0010100010")||
					 instructionCode.substring(0,10).matches("001010100[0-1]")) )
				 actionInFunctionalBlock = new TextLayout(" ", new Font("Verdana", Font.BOLD, 13), frc); 
		 }
		 if(v.isActive() == true){
			 v.setFirst_interaction(false);		 
			 actionInFunctionalBlock.draw(g2d, v.getOppositeAxis(), v.getCurrent());	
			 if (v.getCurrent() == v.getEnd())
				 v.setActive(false);
			 v.setCurrent(v.getCurrent()-1);	
		 }


	 }

	 //convert binnary value to integer.
	 public String parseBinToInt(String code){
		 int value = 0;

		 //if the number is positive
		 if(code.charAt(0)=='0'){
			 for(int i =code.length()-1; i >= 0; i--){
				 if("1".equals(code.substring(i,i+1))){
					 value = value + (int)Math.pow(2,code.length()-i-1);
				 }
			 }
		 }
		 //if the number is negative

		 else {
			 //lowbit 前按位取反
			 int lowbit=-1;
			 for (int i = code.length()-1; i >=0 ; i--) {
				 if(code.charAt(i)=='1'){
					 lowbit=i;
					 break;
				 }
			 }

			 StringBuilder sb=new StringBuilder(code);
			 for (int i = lowbit-1; i >=0; i--) {
				 if(code.charAt(i)=='0')
					 sb.setCharAt(i,'1');
				 else
					 sb.setCharAt(i,'0');
			 }
			 code=sb.toString();

			 for(int i =code.length()-1; i >= 0; i--){
				 if("1".equals(code.substring(i,i+1))){
					 value = value + (int)Math.pow(2,code.length()-i-1);
				 }
			 }
			 value=-value;
		 }
		 return Integer.toString(value);
	 }
	 
	 //set and execute the information about the current position of each line of information in the animation,
	 //verifies the previous status of the animation and increment the position of each line that interconnect the unit function.
	 private void executeAnimation(Graphics g){
		 g2d = (Graphics2D)g; 
		 setUpInstructionInfo(g2d);
		 Vertex vert;
		 for(int i = 0; i < vertexTraversed.size(); i++){
			 vert = vertexTraversed.get(i);
			 //System.out.println(vert.getName());
			 if(vert.isMovingXaxis == true){
				 if(vert.getDirection() == vert.movingLeft){
					 printTrackLtoR(vert);
					 if(vert.isActive() == false){
						 int j = vert.getTargetVertex().size();
						 Vertex tempVertex;
						 for(int k = 0; k < j; k++){
							 tempVertex = outputGraph.get(vert.getNumIndex()).get(k);
							 Boolean hasThisVertex = false;
							 for(int m = 0 ; m < vertexTraversed.size(); m++){
								 if(tempVertex.getNumIndex() == vertexTraversed.get(m).getNumIndex())
									 hasThisVertex = true;
							 }
							 if(hasThisVertex == false){
								 outputGraph.get(vert.getNumIndex()).get(k).setActive(true);
								 vertexTraversed.add( outputGraph.get(vert.getNumIndex()).get(k));
							 } 
						 }
					 }					 
				 }
				 else{
					 printTrackRtoL(vert);
					 if(vert.isActive() == false){
						 int j = vert.getTargetVertex().size();
						 Vertex tempVertex;
						 for(int k = 0; k < j; k++){
							 tempVertex = outputGraph.get(vert.getNumIndex()).get(k);
							 Boolean hasThisVertex = false;
							 for(int m = 0 ; m < vertexTraversed.size(); m++){
								 if(tempVertex.getNumIndex() == vertexTraversed.get(m).getNumIndex())
									 hasThisVertex = true;
							 }
							 if(hasThisVertex == false){
								 outputGraph.get(vert.getNumIndex()).get(k).setActive(true);
								 vertexTraversed.add( outputGraph.get(vert.getNumIndex()).get(k));
							 } 
						 }
					 }
				 }
			 } //end of condition of X axis
			 else{
				 if(vert.getDirection() == vert.movingDownside){
					 if(vert.isText == true)
						 printTextDtoU(vert);	 
					 else
						 printTrackDtoU(vert);
						
						 if(vert.isActive() == false){
							 int j = vert.getTargetVertex().size();
							 Vertex tempVertex;
							 for(int k = 0; k < j; k++){
								 tempVertex = outputGraph.get(vert.getNumIndex()).get(k);
								 Boolean hasThisVertex = false;
								 for(int m = 0 ; m < vertexTraversed.size(); m++){
									 if(tempVertex.getNumIndex() == vertexTraversed.get(m).getNumIndex())
										 hasThisVertex = true;
								 }
								 if(hasThisVertex == false){
									 outputGraph.get(vert.getNumIndex()).get(k).setActive(true);
									 vertexTraversed.add( outputGraph.get(vert.getNumIndex()).get(k));
								 } 
							 }
						 }
					 
				 }
				 else{
					 
					 printTrackUtoD(vert);
					 if(vert.isActive() == false){
						 int j = vert.getTargetVertex().size();
						 Vertex tempVertex;
						 for(int k = 0; k < j; k++){
							 tempVertex = outputGraph.get(vert.getNumIndex()).get(k);
							 Boolean hasThisVertex = false;
							 for(int m = 0 ; m < vertexTraversed.size(); m++){
								 if(tempVertex.getNumIndex() == vertexTraversed.get(m).getNumIndex())
									 hasThisVertex = true;
							 }
							 if(hasThisVertex == false){
								 outputGraph.get(vert.getNumIndex()).get(k).setActive(true);
								 vertexTraversed.add( outputGraph.get(vert.getNumIndex()).get(k));
							 } 
						 }
					 }
				 }
			 }
		 }
	 }

	@Override
	public void mouseClicked(MouseEvent e) {
/*
	 	PointerInfo a = MouseInfo.getPointerInfo();
	 	//limpar a imagem do painel e iniciar o detalhe da unidade funcional.

	 	
 	   if(e.getPoint().getX() > 425 && e.getPoint().getX() < 520 && e.getPoint().getY() > 300 && e.getPoint().getY() < 425){
	 		buildMainDisplayArea("register.png");
	 		FunctionUnitVisualization fu = new FunctionUnitVisualization(instructionBinary, register);
	 		fu.run();
 	   }
 	   
 	   if(e.getPoint().getX() > 355 && e.getPoint().getX() < 415 && e.getPoint().getY() > 180 && e.getPoint().getY() < 280){
 		  buildMainDisplayArea("control.png");
	 		FunctionUnitVisualization fu = new FunctionUnitVisualization(instructionBinary, control);
	 		fu.run();
 	   }
 	   
 	   if(e.getPoint().getX() > 560 && e.getPoint().getX() < 620 && e.getPoint().getY() > 450 && e.getPoint().getY() < 520){
	 		buildMainDisplayArea("ALUcontrol.png");
	 		FunctionUnitVisualization fu = new FunctionUnitVisualization(instructionBinary, aluControl);
	 		fu.run();
	 	}*/
	 	
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}
   }
}
