# **LoongArch汇编教学系统设计文档大纲**  

## 1. 目标描述  
* **兼容性扩展：** 为Mars增添对LoongArch架构的支持，使其能够模拟和执行LoongArch汇编语言编写的程序。满足LoongArch架构教学和开发需求。  
* **指令集实现：** 实现LoongArch指令集的核心部分，包含约50条基础指令。这些指令应该覆盖基本操作，以便用户能够编写和执行基础的汇编程序。  
* **交互式开发体验：** 确保LoongArch架构下的Mars能够提供与MIPS相似的交互式开发体验。这包括代码编辑、调试、性能分析和即时反馈等功能。  
* **教学适用性：** 考虑到Mars在教育领域的广泛应用，新增添的LoongArch支持应该易于理解和使用，以适应不同层次的教学需求。提供相应的教学资源和示例代码，帮助学生更好地学习和掌握LoongArch架构。  
* **代码可移植性：** 鼓励开发者在实现LoongArch支持时考虑到代码的可移植性，使得从MIPS迁移到LoongArch的程序转换尽可能简单，降低学习曲线和迁移成本。   
* **文档和指南：** 提供全面的文档和使用指南，详细描述如何在Mars中使用LoongArch架构，包括快速入门、API参考和常见问题解答等。  

## 2. 比赛题目分析和相关资料调研  
###  比赛题目分析：  
本次比赛的目标是为Mars增添对LoongArch架构的支持，并实现部分或全部LoongArch指令集，以支持Mars的大部分特性。   
###  相关资料调研：
* >《汇编语言（基于 LoongArch 指令系统）》[^1]教材结合 LoongArch 指令集，从计算机系统的角度阐述了汇编语言的理解和应用以及描述了计算机系统的组成及硬、软件的关系。
* >龙芯开源社区[^2]和相关Gitee项目为开发者提供了交流和合作的平台，促进了LoongArch生态的健康发展.
* >LoongArch指令集具有RISC特征，采用定长指令和规整的编码格式，融合了MIPS和RISC-V的优点.
* >为实现LoongArch支持，开发者可以参考LoongArch指令集手册，了解指令集的详细信息和使用方法。

## 3. 系统框架设计  
### 1.设计逻辑框架

![Lars汇编模拟器](.\picture\Lars汇编模拟器.png)

### 2.设计开发框架

​	Lars沿用Mars的java.swing的图形库进行设计，依据该图形库，Lars的开发框架为**MVC（Model View Controller）**设计框架，即将Lars主要分成模型，视图，控制器三部分进行开发。

![image-20240502175204011](.\picture\MVC框架图.png)

### 3.设计物理框架

​	Lars作为本地运行的汇编模拟器，只涉及到用户本身的计算机。

## 4. 开发计划  
### 1.需求分析和设计阶段：
1. 将支持MIPS的汇编模拟器Mars进行底层代码的修改，改成支持LoongArch32位精简指令集的Lars汇编模拟器。
2. 对Mars的部分如数据通路模拟，分支跳转记录等拓展工具进行修改，将其修改成支持LoongArch32位精简指令集的拓展工具。
3. 结合LoongArch32位精简指令集，修改Mars的帮助文档，注释等，便于其对LoongArch的汇编教学。
### 2.核心功能开发阶段：
#### 1.环境搭建：

##### 	1.JDK版本：

	java version "20.0.1" 2023-04-18
	Java(TM) SE Runtime Environment (build 20.0.1+9-29)
	Java HotSpot(TM) 64-Bit Server VM (build 20.0.1+9-29, mixed mode, sharing)
##### 	2.IDE环境：

​		IntelliJ IDEA 2023.1.3

##### 	3.Mars版本:

​	

#### 2.LoongArch32精简指令集支持

##### 1.指令格式:

​		指令集的格式根据LoongArch32精简指令集文档信息进行实现，分为9种典型的指令集格式，但对部分指令格式与典型的指令集格式有所不同，所以在编码的过程中，为其增加额外的指令格式符。

​	

[^LoongArch32位精简指令集的9种典型指令格式]: ![LoongArch32位精简指令集的9种典型指令格式](C:.\picture\9种典型指令格式.png)



[^开发过程中实现的指令格式]: ![image-20240427114150319](.\picture\指令格式实现.png)



##### 2.指令语句

​	对于不同的指令集语句，Lars实现了对其：样例描述、机器码、语句作用、语句格式等性质的描述。

##### 3.指令模拟

Lars沿用了Mars的编译和程序模拟运行的过程，并在其基础上对LoongArch的支持作了修改。

在编译的过程中，编译器会对文本进行拆分，区分.text域的指令和.data域的变量。然后对指令语句进行拆分，对语句的变量，编译器助记符等进行具体意义的转述，将其转换为初步的汇编语句，然后再对初步的汇编语句转换为二进制机器码，最后进行程序的组装和执行。

在指令的模拟过程中，单个指令在编写时就赋予了其功能的模拟过程，例如下图所示的指令	$add.w $

其获取原指令中的rk,rj中的值，然后将rk,rj中的值求和用以更新rd的值。实现了3R求和操作指令。

[^add.w指令的实现]: ![image-20240427115936106](C:.\picture\指令实现样例.png)



##### 4.伪指令实现

基于Mars对Mips的伪指令实现，Lars为了便于程序的开发和LoongArch汇编的教学，也对Lars添加了对伪指令的支持。

Lars的伪指令的具体实现方式是：

​	1.在编译阶段识别伪指令

​	2.对伪指令的操作进行实际指令集指令的拆分

​	3.实现拆分后的指令集指令以实现伪指令

Lars实现了部分汇编程序编写过程中常用的一些伪指令，如

**la**(Load Address)	**li**(Load Immediate)	**move**(Move value of  register 1 to register 2)

**nop**(null instruction)	**jirl**(Return the function call) ...

其中部分伪指令来自于LoongArch官方文档

[^官方文档所提供的伪指令]: ![image-20240427121551399](.\picture\伪指令.png)



部分指令沿用Mars的写法，但对其进行LoongArch化的修改。



#### 3.用户界面开发：

开发用户友好的界面，包括代码编辑器和模拟器控制面板。

### 3.调试和性能优化阶段：

#### 1.调试

##### 1.单元调试

​	对Lars的程序最小可测单位进行单元的调试，对主要修改代码功能的函数方法和类定义进行分析调试，测试代码的鲁棒性和可行性。

##### 2.集成调试

​	在单元调试完成验收的基础上，对各单元的集成单位进行调试。将部分模块进行组合的测试，比如在Lars中，对指令集所相关的指令执行模拟，指令编译，指令编写等模块进行集成的测试，从而发现不同模块中是否存在接口的错误和冲突。

##### 3.系统调试

​	在集成调试完成验收的基础上，对各模块进行组合测试。在本项目中，主要分成前端和后端的系统测试：后端中主要测试汇编程序编译，执行，输出以及硬件的模拟过程代码的测试；前端中主要测试GUI的显示以及前后端接口的测试。

​	在该调试过程中，事先编写一定数量的LoongArch汇编程序进行系统的测试，在测试的过程中关注指令的执行情况，程序的运行状态，实时关注前后端是否接口冲突和错误。

##### 4.用户验收调试

​	在所有代码测试完成的基础上，程序进行试运行，邀请一些学习汇编语言的学生和教师进行试用。在用户的验收反馈中获取程序在代码测试过程中未发现的问题，从而进行及时的修改和更正。

​	

#### 2.性能优化

##### 1.代码优化

​	对程序进行代码层面的优化，减少不必要的运算，运用算法和数据结构以减少内存的分配和时间的占用。

​	比如在指令集的遍历中，可以事先对指令数组进行哈希化，在哈希表中查询指令，可以优化指令的编译速度。

##### 2.并发与并行

​	对程序进行并发和并行的优化

### 4.扩展功能开发阶段：

#### 1.扩展功能分析

Lars的主要扩展功能来自于Mars的tool工具，此工具栏包含多种辅助指令分析和汇编编程的各种扩展工具。Lars的扩展功能开发就是将这些工具进行LoongArch指令集的适配。

扩展功能主要包含以下方面：

1.BHT simulator：

分支跳转模拟器，用于记录分支跳转语句的执行，提供跳转语句的执行地址，目标地址，以及在跳转的过程中目标地址是否被占用，从而记录跳转的正确率。

2.Instruction counter:	

指令计数器，用于记录每种格式指令的执行次数和指令数量占比。

3.Instruction statistics:	

指令统计器，用于记录每种指令所调用主要硬件的数量。

4.Floating point representation:	

浮点数表示器，用于将浮点数显示成32bit的IEEE754标准机器码。

5.LoongArch Xrays:	

指令硬件执行模拟器，用于模拟显示不同类型指令在硬件中执行的数据通路，在与程序连接后，可以给出每一步执行指令的数据通路和指令的机器码拆分。

6.Screen magnifier:	

屏幕放大镜，用于屏幕的放大。



#### 2.扩展功能开发

扩展功能的开发基于Java的swing图形库，并且沿用了AbstractMarsToolAndApplication抽象类作为此类功能与用户编写的LoongArch汇编程序进行接口连接，监听程序执行，使拓展功能工具能够实时根据指令运行而进行相应功能的运行。

##### 1.BHT simulator

在程序运行过程中，读取当前指令，判断指令是否为分支指令，若是分支指令则读取该分支指令的属性，判断分支进行是否执行成功，并且记录该指令的运行。

##### 2.Instruction counter

读取当前指令，根据指令的类型进行计数，并且计算指令在已执行指令中数目的占比，然后显示。

##### 3.Instruction statistics

读取当前指令，根据指令所调用的硬件方式进行计数。

##### 4.Floating point representation

展示浮点数的754表示，显示机器码。

##### 5.LoongArch Xrays:	

读取当前指令，根据当前指令格式和硬件调用情况进行数据通路演示绘图。

绘图数据从xml文件中读取，将LoongArch的数据通路每一条通路作为顶点存储，在绘图过程中根据相应的指令格式选择相应的颜色进行绘图，表示不同路线的选通和执行。



#### 3.扩展功能调试

### 5.需求文档编写阶段：



1.需求文档目标



2.需求文档内容



3.需求文档修正

## 5. 比赛过程中的重要进展  



1.项目确立



2.主要指令集的实现



3.LoongArch程序的可执行



4.扩展功能实现



5.README文档编写

## 6. 系统测试情况  

1.指令集测试



2.程序编写测试



3.前端显示测试



4.运行环境测试

## 7. 遇到的主要问题和解决方法 

1.LoongArch指令的不熟悉



2.Mars编写时代码较为过时的语法修改



3.Mips和LoongArch特性不同导致的冲突

## 8. 分工和协作  

1.指令集相关代码编写



2.前端界面的优化和LoongArch化



3.帮助文档的编写



4.伪指令拓展



5.扩展功能实现

## 9. 提交仓库目录和文件描述  

1.仓库目录



2.文件描述

## 10. 比赛收获
[^1]:汇编语⾔（基于 LoongArch 指令系统）是由清华大学张悠慧教授编写，龙芯中科对本书出版提供相应的支持.
[^2]:龙芯开源社区网址：<http://www.loongnix.cn/>